package com.pointswell.awareim.plugins;

import java.util.Collection;
import java.util.HashSet;

import org.apache.commons.validator.routines.*;

import com.bas.shared.data.FunctionDescription;
import com.bas.shared.functions.IFunctionLibrary;
import com.bas.shared.ruleparser.INodeHelper;
import com.bas.shared.ruleparser.ISQLBuilderHelper;
import com.bas.shared.ruleparser.ParseException;

public class FunctionLibrary implements IFunctionLibrary {

	@Override
	public Object calculate(String functionName, Object[] parameters, INodeHelper arg2) 
			throws ParseException {
		if (functionName.equalsIgnoreCase("VALIDATE_ISBN"))
		{
			if (parameters == null || parameters.length !=1 || !(parameters[0] instanceof String))
				throw new ParseException ("Invalid parameters");
			String inputString = (String) parameters [0];
			ISBNValidator ISBNToCheck = new ISBNValidator ();
			{
				return (ISBNToCheck.isValid(inputString));
			}
		}
		if (functionName.equalsIgnoreCase("CONVERT_ISBN10"))
		{
			if (parameters == null || parameters.length !=1 || !(parameters[0] instanceof String))
				throw new ParseException ("Invalid parameters");
			String inputString = (String) parameters [0];
			ISBNValidator ISBNToCheck = new ISBNValidator ();
			{
				return (ISBNToCheck.convertToISBN13(inputString));
			}
		}
		if (functionName.equalsIgnoreCase("VALIDATE_IBAN"))
		{
			if (parameters == null || parameters.length !=1 || !(parameters[0] instanceof String))
				throw new ParseException ("Invalid parameters");
			String inputString = (String) parameters [0];
			IBANValidator IBANToCheck = new IBANValidator ();
			{
				return (IBANToCheck.isValid(inputString));
			}
		}
		if (functionName.equalsIgnoreCase("VALIDATE_ISSN"))
		{
			if (parameters == null || parameters.length !=1 || !(parameters[0] instanceof String))
				throw new ParseException ("Invalid parameters");
			String inputString = (String) parameters [0];
			ISSNValidator ISSNToCheck = new ISSNValidator ();
			{
				return (ISSNToCheck.isValid(inputString));
			}
		}
		return null;
	}

	@Override
	public FunctionDescription[] getAvailableFunctions() {
	FunctionDescription [] results = new FunctionDescription [4];
		
		results [0] = new FunctionDescription ("VALIDATE_ISBN", "Parameters: Plain Text: ISBN10 or ISBN13 Code.\nReturn: Yes if validation succeded, otherwise No");
		results [1] = new FunctionDescription ("CONVERT_ISBN10", "Parameters: Plain Text: ISBN10.\nReturn: ISBN13 if validation succeded");
		results [2] = new FunctionDescription ("VALIDATE_IBAN", "Parameters: Plain Text: IBAN.\nReturn: Yes if validation succeded, otherwise No");
		results [3] = new FunctionDescription ("VALIDATE_ISSN", "Parameters: Plain Text: ISSN.\nReturn: Yes if validation succeded, otherwise No");				
		return results;
	}

	@Override
	public String getName() {

		return "Function Library";
	}

	@Override
	public HashSet getRequiredFactPatterns(String functionName, Collection params) {
		
		return null;
	}

	@Override
	public Class getTypeClass(String functionName, Collection parameters) {
		if (functionName.equalsIgnoreCase("VALIDATE_ISBN")) 
		{
			return Boolean.class;
			}
		if (functionName.equalsIgnoreCase("CONVERT_ISBN10")) 
		{
			return String.class;
			}
		if (functionName.equalsIgnoreCase("VALIDATE_IBAN")) 
		{
			return String.class;
			}
		if (functionName.equalsIgnoreCase("VALIDATE_ISSN")) 
		{
			return String.class;
			}
		return null;
	}

	@Override
	public boolean hasFunction(String functionName, int paramNmb) {
		if (functionName.equalsIgnoreCase("VALIDATE_ISBN") && paramNmb == 1)
			return true;
		if (functionName.equalsIgnoreCase("CONVERT_ISBN10") && paramNmb == 1)
			return true;
		if (functionName.equalsIgnoreCase("VALIDATE_IBAN") && paramNmb == 1)
			return true;
		if (functionName.equalsIgnoreCase("VALIDATE_ISSN") && paramNmb == 1)
			return true;
		return false;
	}

	@Override
	public String toSQL(String functionName, Collection params, ISQLBuilderHelper arg2) throws Exception {
		return null;
	}

}
